<?php
namespace Teknisa\CMP\Modules\Cadastros\Services;

use PHPExcel_IOFactory;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Cmp00100SheetImporter {
    const TMP_DATA_TABLE = "IMP_GRUPOPRATO";
    const SHEETS_FOLDER = __DIR__ . "/../sheets/";
    const SHEET_TEMPLATE = __DIR__ . "/../sheets/Modelo.xlsx";
    const SHEET_TEMPLATE_MODIFICADO = __DIR__ . "/../sheets/ModeloModificado.xlsx";
    const COLUMNS_MAP = array(
        "CDARVPROD"      => array(
                                "index"     => 0,
                                "maxLength" => 20,
                                "nullable"  => false,
                                "zerofill"  => false,
                                "label"     => "CODIGO PRODUTO"
                            ),
        "NMPRODUTO"    => array(
                                "index"     => 1,
                                "nullable"  => true,
                                "label"     => "NOME PRODUTO"
                            ),
        "VRARREDEMPE" => array(
                                "index"     => 2,
                                "nullable"  => true,
                                "zerofill"  => false,
                                "label"     => "ARRENDONDAMENTO EMP"
                            ),
        "CDEXPRODFORN" => array(
                                "index"     => 3,
                                "maxLength" => 30,
                                "nullable"  => true,
                                "zerofill"  => true,
                                "label"     => "CODIGO EXTERNO"
                            ),
        "VRPERPIPRFO"  => array(
                                "index"     => 4,
                                "nullable"  => true,
                                "zerofill"  => false,
                                "label"     => "PI/DSV"
                            ),
        "VRARREDCMPFO"  => array(
                                "index"     => 5,
                                "nullable"  => true,
                                "nullable"  => false,
                                "label"     => "ARREDONDAMENTO CMP"
                            ),
        "VRPERVARPREPRFO"  => array(
                                "index"     => 6,
                                "nullable"  => true,
                                "zerofill"  => false,
                                "label"     => "VARIACAO"
                            ),
        "NRDIASVALPRFORN" => array(
                                "index"     => 7,
                                "nullable"  => true,
                                "zerofill"  => false,
                                "label"     => "VALIDADE"
                            )
    );
    
    const COLUMNS_MAP_MODIFICADO = array(
        "CDARVPROD"      => array(
                                "index"     => 0,
                                "maxLength" => 20,
                                "nullable"  => false,
                                "zerofill"  => false,
                                "label"     => "CODIGO PRODUTO"
                            ),
        "NMPRODUTO"    => array(
                                "index"     => 1,
                                "nullable"  => true,
                                "label"     => "NOME PRODUTO"
                            ),
        "VRARREDEMPE" => array(
                                "index"     => 2,
                                "nullable"  => true,
                                "zerofill"  => false,
                                "label"     => "ARRENDONDAMENTO EMP"
                            )
    );

    protected $errors = array();
    protected $errorMessages = array();
    protected $nrlogimpgrupoprato;

    public function __construct($crudService, $connection, $environment, $dataSourceManager/*, $restrContrService*/) {
        $this->crudService         = $crudService;
        $this->connection          = $connection;
        $this->environment         = $environment;
        $this->dataSourceManager   = $dataSourceManager;
        //$this->restrContrService = $restrContrService;
    }

    public function getNrlogimpgrupoprato() {
        return $this->nrlogimpgrupoprato;
    }

    public function downloadTemplate(DTO\Request $request, DTO\Response $response) {
        $idsisdos = $this->validaIdsisdos();
        if($idsisdos != 3){
            $name = "Modelo.xls";
            $template = self::SHEET_TEMPLATE;
        }else{
            $name = "ModeloModificado.xls";
            $template = self::SHEET_TEMPLATE_MODIFICADO;
        }
        header('Accept-Ranges: none');
        header("Content-Type: " . "xlsx");
        header("Content-Disposition: attachment; filename=" . $name);
        
        readfile($template);
        
        exit;
    }

    public function createTemporaryFile($sheetImport) {
        if (    isset($sheetImport) && 
                is_array($sheetImport) && 
                !empty($sheetImport) && 
                isset($sheetImport[0]["b64File"]) && 
                isset($sheetImport[0]["name"])
            ) {
            $sheetB64 = $sheetImport[0]["b64File"];
            $sheetName = $sheetImport[0]["name"];
            $sheet = $this->decodeFromBase64($sheetB64);
            $filePath = self::SHEETS_FOLDER . 'tmp-' . $sheetName;
            file_put_contents($filePath, $sheet);
            chmod($filePath, 0644);
            return $filePath;
        } else {
            throw new \Exception ("sheetUploadFailure");
        }
    }

    public function loadSheet($sheetPath) {
        try {
            $objPHPExcel = PHPExcel_IOFactory::load($sheetPath);
            return $objPHPExcel->getActiveSheet();
        } catch( \Excention $e) {
            throw new \Exception("invalidSheet");
            
        }
    }

    public function parseSheet($worksheet) {
        try {
            $rows = array();
            $idsisdos = $this->validaIdsisdos();
            if($idsisdos != 3){
                foreach ($worksheet->getRowIterator() AS $row) {
                    $cellIterator = $row->getCellIterator();
                    $cellIterator->resetEnd('H');
                    $cellIterator->setIterateOnlyExistingCells(FALSE); // This loops through all cells,
                    $cells = [];
                    $hasNotNullCell = false;
                    foreach ($cellIterator as $cell) {
                        $value = $cell->getValue();
                        $hasNotNullCell = $hasNotNullCell || $value;
                        $cells[] = $value;
                    }
                    if ($hasNotNullCell) { // At least one cell with value
                        $rows[] = $cells;
                    }
                }
            }else{
                foreach ($worksheet->getRowIterator() AS $row) {
                    $cellIterator = $row->getCellIterator();
                    $cellIterator->resetEnd('C');
                    $cellIterator->setIterateOnlyExistingCells(FALSE); // This loops through all cells,
                    $cells = [];
                    $hasNotNullCell = false;
                    foreach ($cellIterator as $cell) {
                        $value = $cell->getValue();
                        $hasNotNullCell = $hasNotNullCell || $value;
                        $cells[] = $value;
                    }
                    if ($hasNotNullCell) { // At least one cell with value
                        $rows[] = $cells;
                    }
                }
            }
            
            if (((is_array($rows) && !is_null($rows)) ? count($rows) : 0) < 2) {
                throw new \Exception("emptySheet");
            }
        
            return $rows;
        } catch( \Excention $e) {
            throw $e;
        }
    }

    public function validateSheetColumns($sheet) {
        if (isset($sheet[0])) {
            $header = $sheet[0];
            
            $count = (is_array($header) && !is_null($header)) ? count($header) : 0;
                
            if ($count == 8) {
                foreach (self::COLUMNS_MAP as $column) {
                    if ($header[$column["index"]] != $column["label"]) {
                        throw new \Exception("invalidSheetColumnsLabels");
                    }
                }
            } else{
                if($count == 3){
                    foreach (self::COLUMNS_MAP_MODIFICADO as $column) {
                        if ($header[$column["index"]] != $column["label"]) {
                            throw new \Exception("invalidSheetColumnsLabels");
                        }
                    }
                }else {
                    throw new \Exception("invalidSheetColumnsNumber");
                }
            }
        } else {
            throw new \Exception("invalidSheet");
        }
    }

    public function validateSheetRows(&$sheet, $response) {
        foreach ($sheet as $key => &$row) {
            if ($key != 0) { // Skip header row
                $this->validateEmptyValues($row, $key);
                $this->validateInvalidValues($row, $key);
                
            }
        }
        if (!empty($this->errors)) {
            $this->decodeErrorMessages();
            $response->addDataSet(new DataSet("errors", $this->errorMessages));
            throw new \Exception("sheetErrors", 1);
        }
    }

    private function addError($rowNum, $errorType, $error) {
        if (!isset($this->errors[$rowNum])) {
            $this->errors[$rowNum] = array();
        }
        if (!isset($this->errors[$rowNum][$errorType])) {
            $this->errors[$rowNum][$errorType] = array();
        }
        array_push($this->errors[$rowNum][$errorType], $error);
    }

    private function validateEmptyValues(&$row, $rowNum) {
        $idsisdos = $this->validaIdsisdos();
        if($idsisdos != 3){
            foreach (self::COLUMNS_MAP as $columnName => $column) {
                if (is_null($row[$column["index"]]) && !$column["nullable"]) {
                    $this->addError($rowNum, "emptyCell", $columnName);
                }
            }
        }else{
            foreach (self::COLUMNS_MAP_MODIFICADO as $columnName => $column) {
                if (is_null($row[$column["index"]]) && !$column["nullable"]) {
                    $this->addError($rowNum, "emptyCell", $columnName);
                }
            }
        }
    }

    private function validateInvalidValues(&$row, $rowNum) {
        $idsisdos = $this->validaIdsisdos();
        if($idsisdos != 3){
            foreach (self::COLUMNS_MAP as $columnName => $column) {
                if (strlen($row[$column["index"]]) != 0) {
                    if (isset($column["length"]) && strlen($row[$column["index"]]) != $column["length"]) {
                        if (strlen($row[$column["index"]]) < $column["length"] && isset($column["zerofill"]) && $column["zerofill"]) {
                            $row[$column["index"]] = str_pad($row[$column["index"]], $column["length"], "0", STR_PAD_LEFT);
                        } else {
                            $this->addError($rowNum, "invalidLength", $columnName); 
                        }
                    } else
                    if (isset($column["maxLength"]) && strlen($row[$column["index"]]) > $column["maxLength"]) {
                        $this->addError($rowNum, "invalidMaxLength", $columnName); 
                    }
                }
            }
        }else{
            foreach (self::COLUMNS_MAP_MODIFICADO as $columnName => $column) {
                if (strlen($row[$column["index"]]) != 0) {
                    if (isset($column["length"]) && strlen($row[$column["index"]]) != $column["length"]) {
                        if (strlen($row[$column["index"]]) < $column["length"] && isset($column["zerofill"]) && $column["zerofill"]) {
                            $row[$column["index"]] = str_pad($row[$column["index"]], $column["length"], "0", STR_PAD_LEFT);
                        } else {
                            $this->addError($rowNum, "invalidLength", $columnName); 
                        }
                    } else
                    if (isset($column["maxLength"]) && strlen($row[$column["index"]]) > $column["maxLength"]) {
                        $this->addError($rowNum, "invalidMaxLength", $columnName); 
                    }
                }
            }
        }
    }

    private function decodeErrorMessages() {
        $tab = "          ";
        foreach ($this->errors as $rowNum => $error) {
            $message = "Linha " . ($rowNum + 1) . ":";
            array_push($this->errorMessages, $message);
            if (isset($error["emptyCell"])) {
                foreach (($error["emptyCell"]) as $column) {
                    $message = $tab . "O " . self::COLUMNS_MAP[$column]["label"] . " deve ser informado.";
                    array_push($this->errorMessages, $message);
                }
            }

            if (isset($error["invalidLength"])) {
                foreach (($error["invalidLength"]) as $column) {
                    $message = $tab . "O " . self::COLUMNS_MAP[$column]["label"] . " deve possuir exatamente ". self::COLUMNS_MAP[$column]["length"] . " caracteres.";
                    array_push($this->errorMessages, $message);
                }
            }

            if (isset($error["invalidMaxLength"])) {
                foreach (($error["invalidMaxLength"]) as $column) {
                    $message = $tab . "O " . self::COLUMNS_MAP[$column]["label"] . " deve possuir no máximo ". self::COLUMNS_MAP[$column]["maxLength"] . " caracteres.";
                    array_push($this->errorMessages, $message);
                }
            }

            if (isset($error["filialServicoNotFound"])) {
                $message = $tab . "Unidade/Serviço não encontrado.";
                array_push($this->errorMessages, $message);
            }
            
            if (isset($error["produtoNotFound"])) {
                foreach (($error["produtoNotFound"]) as $column) {
                    $message = $tab . "O " . self::COLUMNS_MAP[$column]["label"] . " informado não foi encontrado";
                    array_push($this->errorMessages, $message);
                }
            }
            
            if (isset($error["arredInvalid"])) {
                foreach (($error["arredInvalid"]) as $column) {
                    $message = $tab . "O " . self::COLUMNS_MAP[$column]["label"] . " deve ser numérico e maior que 0";
                    array_push($this->errorMessages, $message);
                }
            }

        }
    }

    /*private function decodeErrorMessagesFromProcedure($nrlogimpgrupoprato) {
        $conditions = array(
            array(
                "columnName" => "NRLOGIMPGRUPOPRATO",
                "operator"   => "=",
                "value"      => $nrlogimpgrupoprato
            )
        );
        $errors = $this->crudService->getFilteredDataSet("log_imp_grupoprato", $conditions)->getRows();
        usort($errors, function($a, $b) {
			return strnatcmp($a['DSLOGIMPGRUPOPRATO'], $b['DSLOGIMPGRUPOPRATO']);
		});
        foreach ($errors as $error) {
            array_push($this->errorMessages, $error["DSLOGIMPGRUPOPRATO"]);
        }
    }*/

    private function decodeFromBase64($sheet) {
        $b64File = explode(',', $sheet);
        $base64_decode = base64_decode($b64File[1], true);

        return $base64_decode;
    }

    // public function saveSheetData($sheet, $response) {
    //     $rows = $this->buildRows($sheet);
    //     return $this->persistData($rows, $response);
    // }

    // private function buildRows($sheet) {
    //     //$nrimpgrupoprato = $this->restrContrService->getNewCode(self::TMP_DATA_TABLE);
    //     $rows = array();
    //     $now = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
    //     foreach ($sheet as $key => $sheetRow) {
    //         if ($key != 0) { // Skip header row
    //             $row = array(
    //                 "NRIMPGRUPOPRATO" => $nrimpgrupoprato,
    //                 "NRINDICE"        => $key + 1,
    //                 "CDFILIAL"        => $sheetRow[self::COLUMNS_MAP["CDFILIAL"]["index"]],
    //                 "CDSERVFILI"      => $sheetRow[self::COLUMNS_MAP["CDSERVFILI"]["index"]],
    //                 "CDAGRUPGRPRAT"   => $sheetRow[self::COLUMNS_MAP["CDAGRUPAMENTO"]["index"]],
    //                 "NMAGRUPGRPRAT"   => $sheetRow[self::COLUMNS_MAP["NMAGRUPAMENTO"]["index"]],
    //                 "CDGRUPOPRATO"    => $sheetRow[self::COLUMNS_MAP["CDGRUPOPRATO"]["index"]],
    //                 "NMGRUPOPRATO"    => $sheetRow[self::COLUMNS_MAP["NMGRUPOPRATO"]["index"]],
    //                 "CDPRATO"         => $sheetRow[self::COLUMNS_MAP["CDPRATO"]["index"]],
    //                 "NRORG"           => $this->environment->getNrOrg(),
    //                 "IDATIVO"         => "S",
    //                 "DTINCLUSAO"      => $now,
    //                 "DTULTATU"        => $now,
    //                 "NRORGINCLUSAO"   => $this->environment->getNrOrg(),
    //                 "CDOPERINCLUSAO"  => $this->environment->getCdOperador(),
    //                 "NRORGULTATU"     => $this->environment->getNrOrg(),
    //                 "CDOPERULTATU"    => $this->environment->getCdOperador()
    //             );
    //             array_push($rows, $row);
    //         }
    //     }
    //     return $rows; 
    // }

    // private function persistData($rowsToSave, $response) {
    //     $nrgrupoprato = $rowsToSave[0]["NRIMPGRUPOPRATO"];
    //     $this->connection->beginTransaction();
    //     foreach ($rowsToSave as $row) {
    //         try {
    //             $this->insertImpgrupoprato($row);
    //         } catch (\Exception $e) {
    //             if (strrpos($e->getMessage(), "ORA-02291")) {
    //                 $this->addError($row["NRINDICE"] - 1, "filialServicoNotFound", true);
    //             } else {
    //                 throw $e;
    //             }
    //         }
    //     }
    //     if (!empty($this->errors)) {
    //         $this->decodeErrorMessages();
    //         $response->addDataSet(new DataSet("errors", $this->errorMessages));
    //         $this->connection->rollBack();
    //         throw new \Exception("sheetErrors'", 1);
    //     }
    //     $this->connection->commit();
    //     return $nrgrupoprato;
    // }

    // private function insertImpgrupoprato($row) {
    //     $sql = "INSERT INTO IMP_GRUPOPRATO 
    //                 (NRIMPGRUPOPRATO, NRINDICE, CDFILIAL, CDSERVFILI, CDAGRUPGRPRAT, NMAGRUPGRPRAT, CDGRUPOPRATO, NMGRUPOPRATO, 
    //                 CDPRATO, NRORG, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU)
    //             VALUES                      
    //                 (:NRIMPGRUPOPRATO, :NRINDICE, :CDFILIAL, :CDSERVFILI, :CDAGRUPGRPRAT, :NMAGRUPGRPRAT, :CDGRUPOPRATO, :NMGRUPOPRATO,
    //                 :CDPRATO, :NRORG, :IDATIVO, :DTINCLUSAO, :DTULTATU, :NRORGINCLUSAO, :CDOPERINCLUSAO, :NRORGULTATU, :CDOPERULTATU)
    //             ";

    //     $query = $this->connection->prepare($sql);
    //     $query->bindValue("NRIMPGRUPOPRATO", $row['NRIMPGRUPOPRATO']);
    //     $query->bindValue("NRINDICE", $row['NRINDICE']);
    //     $query->bindValue("CDFILIAL", $row['CDFILIAL']);
    //     $query->bindValue("CDSERVFILI", $row['CDSERVFILI']);
    //     $query->bindValue("CDAGRUPGRPRAT", $row['CDAGRUPGRPRAT']);
    //     $query->bindValue("NMAGRUPGRPRAT", $row['NMAGRUPGRPRAT']);
    //     $query->bindValue("CDGRUPOPRATO", $row['CDGRUPOPRATO']);
    //     $query->bindValue("NMGRUPOPRATO", $row['NMGRUPOPRATO']);
    //     $query->bindValue("CDPRATO", $row['CDPRATO']);
    //     $query->bindValue("NRORG", $row['NRORG']);
    //     $query->bindValue("IDATIVO", $row['IDATIVO']);
    //     $query->bindValue("DTINCLUSAO", $row['DTINCLUSAO']);
    //     $query->bindValue("DTULTATU", $row['DTULTATU']);
    //     $query->bindValue("NRORGINCLUSAO", $row['NRORGINCLUSAO']);
    //     $query->bindValue("CDOPERINCLUSAO", $row['CDOPERINCLUSAO']);
    //     $query->bindValue("NRORGULTATU", $row['NRORGULTATU']);
    //     $query->bindValue("CDOPERULTATU", $row['CDOPERULTATU']);
    //     $query->execute();
    // }

    // public function executeProcedure($nrimpgrupoprato, $response) {
    //     $procedure = new StoredProcedure($this->connection, 'IMPORTA_GRUPOPRATO');
    //     $procedure->addParam(new Param('P_NRIMPGRUPOPRATO', Param::PARAM_INPUT, $nrimpgrupoprato));
    //     $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $this->environment->getNrOrg()));
    //     $procedure->addParam(new Param('P_RETORNO', Param::PARAM_OUTPUT, NULL, NULL, 20));
    //     $this->nrlogimpgrupoprato = $procedure->execute()["P_RETORNO"];
        
    //     if (!is_null($this->nrlogimpgrupoprato)) {
    //         $this->decodeErrorMessagesFromProcedure($this->nrlogimpgrupoprato);
    //         $response->addDataSet(new DataSet("errors", $this->errorMessages));
    //         throw new \Exception("procedureErrors", 1);
    //     }
    // }

    public function removeTemporaryFile($filePath) {
        unlink($filePath);
    }

    // public function removeSheetTmpData($nrimpgrupoprato) {
    //     $sql = "DELETE FROM IMP_GRUPOPRATO WHERE NRIMPGRUPOPRATO = :NRIMPGRUPOPRATO AND NRORG = :NRORG";

    //     $query = $this->connection->prepare($sql);

    //     $query->bindValue("NRIMPGRUPOPRATO", $nrimpgrupoprato);
    //     $query->bindValue("NRORG", $this->environment->getNrOrg());
    //     $query->execute();
    // }

    // public function removeSheetLog($nrlogimpgrupoprato) {
    //     $sql = "DELETE FROM LOG_IMP_GRUPOPRATO WHERE NRLOGIMPGRUPOPRATO = :NRLOGIMPGRUPOPRATO AND NRORG = :NRORG";

    //     $query = $this->connection->prepare($sql);

    //     $query->bindValue("NRLOGIMPGRUPOPRATO", $nrlogimpgrupoprato);
    //     $query->bindValue("NRORG", $this->environment->getNrOrg());
    //     $query->execute();
    // }

    public function gerProdArray($sheet){
        $prodArray = [];
        foreach ($sheet as $ind => $prods) {
            if($ind != 0){
                array_push($prodArray, $prods);
            }
        }
        return $prodArray;
    }
    
    public function validateImportedProd($prodData, $response){
        foreach ($prodData as $key => $prod) {
            switch ($key) {
                case 0:
                    $cdProd = $this->verifyProdExists($prod);
                    if(empty($cdProd)) $this->addError($key, "produtoNotFound", 'CDARVPROD');
                    break;
                case 2:
                    if(!is_numeric ($prod) && $prod <= 0) $this->addError($key, "arredInvalid", 'VRARREDEMPE');
                    break;
                case 4:
                    if(!is_numeric ($prod) && $prod <= 0) $this->addError($key, "arredInvalid", 'VRPERPIPRFO');
                    break;
                case 5:
                    if(!is_numeric ($prod) && $prod <= 0) $this->addError($key, "arredInvalid", 'VRARREDCMPFO');
                    break;
                case 6:
                    if(!is_numeric ($prod) && $prod <= 0) $this->addError($key, "arredInvalid", 'VRPERVARPREPRFO');
                    break;
                case 7:
                    if(!is_numeric ($prod) && $prod <= 0) $this->addError($key, "arredInvalid", 'NRDIASVALPRFORN');
                    break;
            }
            
        }
        if (!empty($this->errors)) {
            $this->decodeErrorMessages();
            $response->addDataSet(new DataSet("errors", $this->errorMessages));
            throw new \Exception("sheetErrors", 1);
        }
    }
    
    public function getCdProd($prodArray, $row){
        foreach ($prodArray as $key => $prod){
            $cdProd = $this->queryProdForn($row, $prod);
            if(empty($cdProd)){
                $codProd = $this->verifyProdExists($prod);
                $this->insertProduct($row, $codProd[0]);
                $this->procInsertProduto($row, $prod, $codProd[0]);
                $cdProd = $this->queryProdForn($row, $prod);
            }
            $this->updateProduct($row, $prod, $cdProd[0]);
        }
    }

    public function validateProdForn($row, $sheet, $response){
        $prodArray = self::gerProdArray($sheet);
        self::validateImportedProd($prodArray, $response);
        self::getCdProd($prodArray, $row);
    }
    
    public function verifyProdExists($prodData){
        $param = array();
        $param['CDARVPROD'] = $prodData[0];
        $query = "  SELECT
                        CDPRODUTO
                    FROM
                        PRODUTO
                    WHERE
                        CDARVPROD = :CDARVPROD ";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw new \Exception("invalidSheet");
        }
    }
    
    public function queryProdForn($row, $prod){
        $param = array();
        $param['P_CDFORNECED'] = $row['CDFORNECED'];
        $param['P_CDARVPROD']  = $prod[0];
        $query = "  SELECT
                        PRODFORN.CDPRODUTO,
                        PRODUTO.CDPRODUTO AS CDPRODCOMPL
                    FROM
                        PRODFORN,
                        PRODUTO
                    WHERE
                        PRODFORN.CDPRODUTO = PRODUTO.CDPRODUTO AND
                        PRODFORN.CDFORNECED = :P_CDFORNECED AND
                        ((PRODUTO.CDARVPROD LIKE :P_CDARVPROD || '%') OR
                        (INSTR(:P_CDARVPROD, PRODUTO.CDARVPROD) = 1)) ";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw new \Exception("invalidSheet");
        }
    }
    
    public function insertProduct($row, $cdProd){
        $param = array();
        $param['P_CDFORNECED'] = $row['CDFORNECED'];
        $param['P_CDPRODUTO']  = $cdProd['CDPRODUTO'];
        $query = "  INSERT INTO PRODFORN 
                        (CDFORNECED,CDPRODUTO,IDCOTAOBRIG)
                    VALUES
                        (:P_CDFORNECED, :P_CDPRODUTO, 'N') ";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw new \Exception("invalidSheet");
        }
    }
    
    public function updateProduct($row, $prod, $cdProd){
        $param = array();
        $param['CDFORNECED']      = $row['CDFORNECED'];
        $param['CDPRODUTO']       = $cdProd['CDPRODUTO'];
        $param['CDPRODCOMPL']     = $prod[0];
        $param['VRARREDEMPE']     = $prod[2];
        
        $idsisdos = $this->validaIdsisdos();
        if($idsisdos != 3){
            $param['CDEXPRODFORN']    = $prod[3];
            $param['VRPERPIPRFO']     = $prod[4];
            $param['VRARREDCMPFO']    = $prod[5];
            $param['VRPERVARPREPRFO'] = $prod[6];
            $param['NRDIASVALPRFORN'] = $prod[7];
            
            $query = "  UPDATE PRODFORNCOMP
                        SET
                            VRARREDCMPFO = DECODE(:VRARREDCMPFO,0,VRARREDCMPFO,:VRARREDCMPFO),
                            VRPERPIPRFO = DECODE(:VRPERPIPRFO,0,VRPERPIPRFO,:VRPERPIPRFO),
                            VRARREDEMPE = DECODE(:VRARREDEMPE,0,VRARREDEMPE,:VRARREDEMPE),
                            VRPERVARPREPRFO = DECODE(:VRPERVARPREPRFO,0,VRPERVARPREPRFO,:VRPERVARPREPRFO),
                            NRDIASVALPRFORN = DECODE(:NRDIASVALPRFORN,0,NRDIASVALPRFORN,:NRDIASVALPRFORN),
                            CDEXPRODFORN = DECODE(:CDEXPRODFORN,'',CDEXPRODFORN,:CDEXPRODFORN)
                        WHERE
                            CDFORNECED =:CDFORNECED
                            AND   CDPRODUTO =:CDPRODUTO
                            AND   CDPRODCOMPL =:CDPRODCOMPL ";
            try {
                $std = $this->connection->prepare($query);
                $std->execute($param);
            } catch (Exception $e){
                throw new \Exception("invalidSheet");
            }
        }else{
            $query = "  UPDATE PRODFORNCOMP
                        SET
                            VRARREDEMPE = DECODE(:VRARREDEMPE,0,VRARREDEMPE,:VRARREDEMPE)
                        WHERE
                            CDFORNECED =:CDFORNECED
                            AND   CDPRODUTO =:CDPRODUTO
                            AND   CDPRODCOMPL =:CDPRODCOMPL ";
            try {
                $std = $this->connection->prepare($query);
                $std->execute($param);
            } catch (Exception $e){
                throw new \Exception("invalidSheet");
            }
        }
    }
    
    public function procInsertProduto($row, $prod, $codProd){
         
        $connection = $this->connection;
        $procedure = new StoredProcedure($connection, 'INSERE_PRODFORNCMP_MARCFORNCMP');
        $procedure->addParam(new Param('P_CDFORNECED',  Param::PARAM_INPUT, $row['CDFORNECED'],     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDMARCA',     Param::PARAM_INPUT, '',                     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODUTO',   Param::PARAM_INPUT, $codProd['CDPRODUTO'],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDARVPROD',   Param::PARAM_INPUT, $prod[0],     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ARRED',       Param::PARAM_INPUT, null,                   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFORNORIG',  Param::PARAM_INPUT, null,                   Param::PARAM_TYPE_STR));
        $rowsaffect = $procedure->execute();
    }
    
    
    public function validaIdsisdos(){
        //$param = array();
        $query = "  SELECT IDSISDOS
                        FROM PARAMGERAL ";
        try {
            $std = $this->connection->prepare($query);
            $std->execute();
            return $std->fetchAll()[0]['IDSISDOS'];
        } catch (Exception $e){
            throw new \Exception("invalidSheet");
        }
    }
}
