<?php
namespace Teknisa\CMP\Modules\Fornecimento\Modules\EmissaoAutorizacaoFornecimento\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class Cmp07000RelacaoFornecedoresAFAnalise extends \Zeedhi\Framework\Controller\Report {
    protected $entityManager;
    protected $qr2Strategy;
    protected $stNomeRel;
    protected $filialService;
    protected $environment;

    public function __construct($reportService, $connection, $entityManager, $qr2Strategy, $birtStrategy, $filialService, $environment) {
        $this->reportService       = $reportService;
        $this->connection          = $connection;
        $this->entityManager       = $entityManager;
        $this->qr2Strategy         = $qr2Strategy;
        $this->birtStrategy        = $birtStrategy;
        $this->filialService       = $filialService;
        $this->environment         = $environment;
    }

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        ini_set('default_socket_timeout', 10000);
        $row = $request->getRow();
        
        $cdOperador = $this->environment->getCdOperador();
        
        if($row["CDFILIAL"] == "T") {
            $newFilial = $this->filialService->retornaListaDeFiliais($cdOperador);
            if(is_array($newFilial)) {
                    $str = "_";
                    foreach($newFilial as $filial) {
                        foreach($filial as $cdfilial) {
                            $str .= $cdfilial . '_';
                        }
                    }
                    $newFilial = $str;
                    $row["CDFILIAL"] = $newFilial;
                }
        }
        
        $this->qr2Strategy->setQr2Type(strtolower($row['QR2TYPE'])); 
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        
        $this->stNomeRel = $this->getReportName($row);
        $this->reportMapping[$this->stNomeRel] =
            array(
                'strategy' => $row['STRATEGY'],
                'parameterMapping' =>
                array(
                    'CDFILIAL'         => 'P_CDFILIAL',
                    'CDFORNECED'       => 'P_CDFORNECED',
                    'CDOPERADOR'       => 'P_CDOPERADOR',
                    'CDREGIAO'         => 'P_CDREGIAO',
                    'CDGRUPFILI'       => 'P_CDGRUPFILI',
                    'COINCIDE'         => 'P_COINCIDE',
                    'DTCONAPUPRECFIN'  => 'P_DTCONAPUPRECFIN',
                    'DTCONAPUPRECINI'  => 'P_DTCONAPUPRECINI',
                    'DTFINAPUPREC'     => 'P_DTFINAPUPREC',
                    'DTFINPER'         => 'P_DTFINPER',
                    'DTINIAPUPREC'     => 'P_DTINIAPUPREC',
                    'DTINIPER'         => 'P_DTINIPER',
                    'IDBAIXAAF'        => 'P_IDBAIXAAF',
                    'LISTASUBGRUP'     => 'P_LISTASUBGRUP',
                    'NACCONSIDERA'     => 'P_NACCONSIDERA',
                    'NMPRODFIN'        => 'P_NMPRODFIN',
                    'NMPRODIN'         => 'P_NMPRODIN',
                    'NRORG'            => 'P_NRORG',
                    'NRSEQAPURAF'      => 'P_NRSEQAPURAF',
                    'PRODFIN'          => 'P_PRODFIN',
                    'PRODINI'          => 'P_PRODINI',
                    'PRODUTFIN'        => 'P_PRODUTFIN',
                    'PRODUTINI'        => 'P_PRODUTINI',
                    'TODOSCON'         => 'P_TODOSCON',
                    'TODOSGER'         => 'P_TODOSGER',
                    'TODOSGRUPFILI'    => 'L_TODOSGRUPFILI',
                    'TODOSPROD'        => 'L_TODOSPROD',
                    'TPENVIOAF'        => 'P_TPENVIOAF',
                    'TPRODUTO'         => 'P_TPRODUTO',
               
                    'CDREGIAO'            =>'L_CDREGIAO',
                    'CDREGIAO1'           =>'L_CDREGIAO1',
                    'CDFILIAL'            =>'L_CDFILIAL',
                    'CDFILIAL1'           =>'L_CDFILIAL1',
                    'CDOPERADOR'          =>'L_CDOPERADOR',
                    'CDOPERADOR1'         =>'L_CDOPERADOR1',
                    'CDFORNECED'          =>'L_CDFORNECED',
                    'CDFORNECED1'         =>'L_CDFORNECED1',
                    'LISTASUBGRUP'        =>'P_LISTASUBGRUP',
                    'LISTASUBGRUP1'       =>'P_LISTASUBGRUP1',
                    'NRSEQAPURAF1'        =>'L_NRSEQAPURAF1',
                    'COINCIDE'            =>'P_COINCIDE',
                    'COINCIDE1'           =>'P_COINCIDE1',
                    'IDBAIXAAF'           =>'P_IDBAIXAAF',
                    'IDBAIXAAF1'          =>'P_IDBAIXAAF1',
                    'TPENVIOAF1'          =>'P_TPENVIOAF1',
                    'NMPRODIN'            =>'P_NMPRODIN',
                    'NMPRODFIN'           =>'P_NMPRODFIN',
                    'NACCONSIDERA'        =>'P_NACCONSIDERA',
                    'NACCONSIDERA1'       =>'P_NACCONSIDERA1',
                    'TODOSGRUPFILI1'      =>'L_TODOSGRUPFILI1',
                    'TODOSGER1'           =>'P_TODOSGER1',
                    'DTINIPER'            =>'P_DTINIPER',
                    'DTFINPER'            =>'P_DTFINPER',
                    'DTINIPER1'           =>'P_DTINIPER1',
                    'DTFINPER1'           =>'P_DTFINPER1',
                    'DTINIAPUPREC'        =>'P_DTINIAPUPREC',
                    'DTFINAPUPREC'        =>'P_DTFINAPUPREC',
                    'DTINIAPUPREC1'       =>'P_DTINIAPUPREC1',
                    'DTFINAPUPREC1'       =>'P_DTFINAPUPREC1',
                    'TODOSCON1'           =>'P_TODOSCON1',
                    'DTCONAPUPRECINI'     =>'P_DTCONAPUPRECINI',
                    'DTCONAPUPRECFIN'     =>'P_DTCONAPUPRECFIN',
                    'DTCONAPUPRECINI1'    =>'P_DTCONAPUPRECINI1',
                    'DTCONAPUPRECFIN1'    =>'P_DTCONAPUPRECFIN1',
                    'TODOSPROD1'          =>'L_TODOSPROD1',
               
                    'NMPRODIN'        => 'P_NMPRODIN',
                    'NMPRODFIN'       => 'P_NMPRODFIN',
                    'DTINIPER'        => 'P_DTINIPER',
                    'DTFINPER'        => 'P_DTFINPER',
                    'IDBAIXAAF'       => 'P_IDBAIXAAF',
                    'DTINIAPUPREC'    => 'P_DTINIAPUPREC',
                    'COINCIDE'        => 'P_COINCIDE',
                    'DTFINAPUPREC'    => 'P_DTFINAPUPREC',
                    'DTCONAPUPRECINI' => 'P_DTCONAPUPRECINI',
                    'DTCONAPUPRECFIN' => 'P_DTCONAPUPRECFIN',
                    'NACCONSIDERA'    => 'P_NACCONSIDERA',
                    'LISTASUBGRUP'    => 'P_LISTASUBGRUP',
                ),
            );
    }

    public function getCodigoGrupProd(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        $params = array(
    		"P_CDFILCENTCMP" => $row['CDFILCENTCMP']
    	);
    	
    	$sql = "SELECT CDGRUPPROD FROM PARASUPRI WHERE CDFILIAL =:P_CDFILCENTCMP";
    	$query = $this->connection->prepare($sql);
    	$query->execute($params);
    	$result = $query->fetchAll();
    	
    	$response->addDataSet(new DataSet('data', [$result[0]]));
    }
}
